@@ -26,16 +26,16 @@ from dogpile.cache import make_region
 from dogpile.cache.api import NoValue
 from sqlalchemy import func
 
-from rucio.common.exception import ConfigNotFound
 from rucio.common.config import config_get
+from rucio.common.exception import ConfigNotFound
 from rucio.db.sqla import models
 from rucio.db.sqla.session import read_session, transactional_session
 
 
 REGION = make_region().configure('dogpile.cache.memcached',
                                  expiration_time=3600,
-                                 arguments={'url': config_get('cache', 'url', False, '127.0.0.1:11211'), 'distributed_lock': True})
-
+                                 arguments={'url': config_get('cache', 'url', False, '127.0.0.1:11211',
+                                                              check_config_table=False), 'distributed_lock': True})
 SECTIONS_CACHE_KEY = 'sections'
 
 
