@@ -25,6 +25,7 @@
 # - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 
 import json
+from configparser import NoOptionError, NoSectionError
 
 from sqlalchemy import or_, delete, update
 from sqlalchemy.exc import IntegrityError
@@ -33,9 +34,8 @@ from dogpile.cache import make_region
 from dogpile.cache.api import NO_VALUE
 
 from rucio.common.config import config_get
-from rucio.common.exception import InvalidObject, RucioException, ConfigNotFound
+from rucio.common.exception import InvalidObject, RucioException
 from rucio.common.utils import APIEncoder
-from rucio.core.config import get
 from rucio.db.sqla import filter_thread_work
 from rucio.db.sqla.models import Message, MessageHistory
 from rucio.db.sqla.session import transactional_session
@@ -62,8 +62,8 @@ def add_message(event_type, payload, session=None):
     services_list = REGION.get('services_list')
     if services_list == NO_VALUE:
         try:
-            services_list = get('hermes', 'services_list')
-        except ConfigNotFound:
+            services_list = config_get('hermes', 'services_list')
+        except (NoOptionError, NoSectionError, RuntimeError):
             services_list = None
         REGION.set('services_list', services_list)
 
@@ -135,15 +135,15 @@ def retrieve_messages(bulk=1000, thread=None, total_threads=None, event_type=Non
 
         # Step 3:
         # Assemble message object
-        for id, created_at, event_type, payload, services in query:
-            message = {'id': id,
+        for id_, created_at, event_type, payload, services in query:
+            message = {'id': id_,
                        'created_at': created_at,
                        'event_type': event_type,
                        'services': services}
 
             # Only switch SQL context when necessary
             if payload == 'nolimit':
-                nolimit_query = session.query(Message.payload_nolimit).filter(Message.id == id).one()[0]
+                nolimit_query = session.query(Message.payload_nolimit).filter(Message.id == id_).one()[0]
                 message['payload'] = json.loads(str(nolimit_query))
             else:
                 message['payload'] = json.loads(str(payload))
