@@ -43,12 +43,16 @@ from __future__ import division
 
 import datetime
 import logging
+import os
+import socket
+import threading
 import time
 
-from rucio.common.config import config_get_bool
+from rucio.common.config import config_get
 from rucio.common.exception import (InvalidRSEExpression, TransferToolTimeout, TransferToolWrongAnswer, RequestNotFound,
                                     DuplicateFileTransferSubmission, VONotFound)
-from rucio.core import request, transfer as transfer_core
+from rucio.common.logging import formatted_logger
+from rucio.core import heartbeat, request, transfer as transfer_core
 from rucio.core.monitor import record_counter, record_timer
 from rucio.core.rse import list_rses
 from rucio.core.rse_expression_parser import parse_expression
@@ -57,6 +61,50 @@ from rucio.db.sqla.constants import RequestState
 from rucio.rse import rsemanager as rsemgr
 
 
+class HeartbeatHandler:
+    """
+    Simple contextmanager which sets a heartbeat and associated logger on entry and cleans up the heartbeat on exit.
+    """
+
+    def __init__(self, executable, logger_prefix=None):
+        self.executable = executable
+        self.logger_prefix = logger_prefix or executable
+
+        self.hostname = socket.getfqdn()
+        self.pid = os.getpid()
+        self.hb_thread = threading.current_thread()
+
+        self.logger = None
+        self.last_heart_beat = None
+
+    def __enter__(self):
+        heartbeat.sanity_check(executable=self.executable, hostname=self.hostname)
+        self.live()
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        if self.last_heart_beat:
+            heartbeat.die(self.executable, self.hostname, self.pid, self.hb_thread)
+            if self.logger:
+                self.logger(logging.INFO, 'Heartbeat cleaned up')
+
+    def live(self, older_than=None):
+        if older_than:
+            self.last_heart_beat = heartbeat.live(self.executable, self.hostname, self.pid, self.hb_thread, older_than=older_than)
+        else:
+            self.last_heart_beat = heartbeat.live(self.executable, self.hostname, self.pid, self.hb_thread)
+
+        prefix = '%s[%i/%i]: ' % (self.logger_prefix, self.last_heart_beat['assign_thread'], self.last_heart_beat['nr_threads'])
+        self.logger = formatted_logger(logging.log, prefix + '%s')
+
+        if not self.last_heart_beat:
+            self.logger(logging.DEBUG, 'First heartbeat set')
+        else:
+            self.logger(logging.DEBUG, 'Heartbeat renewed')
+
+        return self.last_heart_beat, self.logger
+
+
 def submit_transfer(transfertool_obj, transfers, job_params, submitter='submitter', timeout=None, logger=logging.log):
     """
     Submit a transfer or staging request
@@ -123,9 +171,9 @@ def _submit_transfers(transfertool_obj, transfers, job_params, submitter='submit
     if eid is not None:
         duration = time.time() - start_time
         logger(logging.INFO, 'Submit job %s to %s in %s seconds' % (eid, transfertool_obj, duration))
-        record_timer('daemons.conveyor.%s.submit_bulk_transfer.per_file' % submitter, (time.time() - start_time) * 1000 / len(transfers) or 1)
+        record_timer('daemons.conveyor.{submitter}.submit_bulk_transfer.per_file', (time.time() - start_time) * 1000 / len(transfers) or 1, labels={'submitter': submitter})
         record_counter('daemons.conveyor.{submitter}.submit_bulk_transfer', delta=len(transfers), labels={'submitter': submitter})
-        record_timer('daemons.conveyor.%s.submit_bulk_transfer.files' % submitter, len(transfers))
+        record_timer('daemons.conveyor.{submitter}.submit_bulk_transfer.files', len(transfers), labels={'submitter': submitter})
 
     if state_to_set:
         try:
@@ -155,7 +203,7 @@ def get_conveyor_rses(rses=None, include_rses=None, exclude_rses=None, vos=None,
     :param logger:        Optional decorated logger that can be passed from the calling daemons or servers.
     :return:              List of working rses
     """
-    multi_vo = config_get_bool('common', 'multi_vo', raise_exception=False, default=False)
+    multi_vo = config_get('common', 'multi_vo', raise_exception=False, default=False)
     if not multi_vo:
         if vos:
             logger(logging.WARNING, 'Ignoring argument vos, this is only applicable in a multi-VO setup.')
@@ -180,7 +228,7 @@ def get_conveyor_rses(rses=None, include_rses=None, exclude_rses=None, vos=None,
     if include_rses:
         for vo in vos:
             try:
-                parsed_rses = parse_expression(include_rses, filter={'vo': vo}, session=None)
+                parsed_rses = parse_expression(include_rses, filter_={'vo': vo}, session=None)
             except InvalidRSEExpression:
                 logger(logging.ERROR, "Invalid RSE exception %s to include RSEs", include_rses)
             else:
