@@ -26,6 +26,7 @@
 # - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2020-2021
 # - Eric Vaandering <ewv@fnal.gov>, 2021
 # - David Población Criado <david.poblacion.criado@cern.ch>, 2021
+# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
 
 '''
 Dark Reaper is a daemon to manage quarantined file deletion.
@@ -47,7 +48,7 @@ from rucio.common.config import config_get_bool
 from rucio.common.exception import (SourceNotFound, DatabaseException, ServiceUnavailable,
                                     RSEAccessDenied, ResourceTemporaryUnavailable,
                                     RSENotFound, VONotFound)
-from rucio.common.logging import setup_logging
+from rucio.common.logging import setup_logging, formatted_logger
 from rucio.common.utils import daemon_sleep
 from rucio.core.heartbeat import live, die, sanity_check
 from rucio.core.message import add_message
@@ -90,8 +91,9 @@ def reaper(rses, worker_number=0, total_workers=1, chunk_size=100, once=False, s
             # heartbeat
             heartbeat = live(executable=executable, hostname=hostname, pid=pid, thread=thread,
                              hash_executable=hash_executable)
-            logging.info('Dark Reaper({0[worker_number]}/{0[total_workers]}): Live gives {0[heartbeat]}'
-                         .format(locals()))
+            prepend_str = 'dark-reapter [%i/%i] : ' % (heartbeat['assign_thread'], heartbeat['nr_threads'])
+            logger = formatted_logger(logging.log, prepend_str + '%s')
+            logger(logging.INFO, 'Live gives {0[heartbeat]}'.format(locals()))
             nothing_to_do = True
             start_time = time.time()
 
@@ -120,13 +122,11 @@ def reaper(rses, worker_number=0, total_workers=1, chunk_size=100, once=False, s
                                                                    'path': replica['path']}],
                                                             operation='delete',
                                                             scheme=scheme).values())[0])
-                            logging.info('Dark Reaper %s-%s: Deletion ATTEMPT of %s:%s as %s on %s',
-                                         worker_number, total_workers, scope, replica['name'], pfn, rse)
+                            logger(logging.INFO, 'Deletion ATTEMPT of %s:%s as %s on %s', scope, replica['name'], pfn, rse)
                             start = time.time()
                             prot.delete(pfn)
                             duration = time.time() - start
-                            logging.info('Dark Reaper %s-%s: Deletion SUCCESS of %s:%s as %s on %s in %s seconds',
-                                         worker_number, total_workers, scope, replica['name'], pfn, rse, duration)
+                            logger(logging.INFO, 'Deletion SUCCESS of %s:%s as %s on %s in %s seconds', scope, replica['name'], pfn, rse, duration)
                             payload = {'scope': scope,
                                        'name': replica['name'],
                                        'rse': rse,
@@ -141,14 +141,14 @@ def reaper(rses, worker_number=0, total_workers=1, chunk_size=100, once=False, s
                             add_message('deletion-done', payload)
                             deleted_replicas.append(replica)
                         except SourceNotFound:
-                            err_msg = ('Dark Reaper %s-%s: Deletion NOTFOUND of %s:%s as %s on %s'
-                                       % (worker_number, total_workers, scope, replica['name'], pfn, rse))
-                            logging.warning(err_msg)
+                            err_msg = ('Deletion NOTFOUND of %s:%s as %s on %s'
+                                       % (scope, replica['name'], pfn, rse))
+                            logger(logging.WARNING, err_msg)
                             deleted_replicas.append(replica)
                         except (ServiceUnavailable, RSEAccessDenied, ResourceTemporaryUnavailable) as error:
-                            err_msg = ('Dark Reaper %s-%s: Deletion NOACCESS of %s:%s as %s on %s: %s'
-                                       % (worker_number, total_workers, scope, replica['name'], pfn, rse, str(error)))
-                            logging.warning(err_msg)
+                            err_msg = ('Deletion NOACCESS of %s:%s as %s on %s: %s'
+                                       % (scope, replica['name'], pfn, rse, str(error)))
+                            logger(logging.WARNING, err_msg)
                             payload = {'scope': scope,
                                        'name': replica['name'],
                                        'rse': rse,
@@ -176,7 +176,7 @@ def reaper(rses, worker_number=0, total_workers=1, chunk_size=100, once=False, s
                 break
 
             if nothing_to_do:
-                logging.info('Dark Reaper %s-%s: Nothing to do', worker_number, total_workers)
+                logger(logging.INFO, 'Nothing to do')
                 daemon_sleep(start_time=start_time, sleep_time=sleep_time, graceful_stop=GRACEFUL_STOP)
 
         except DatabaseException as error:
