@@ -31,13 +31,15 @@ from urllib.parse import urlencode, urlparse, parse_qsl, urlunparse
 import pytest
 
 import rucio.daemons.reaper.reaper
+from rucio.common.types import InternalAccount
+from rucio.common.utils import generate_uuid
 from rucio.common.exception import ReplicaNotFound, RequestNotFound
+from rucio.core import config as core_config
 from rucio.core import distance as distance_core
 from rucio.core import replica as replica_core
 from rucio.core import request as request_core
 from rucio.core import rse as rse_core
 from rucio.core import rule as rule_core
-from rucio.core import transfer as transfer_core
 from rucio.daemons.conveyor.finisher import finisher
 from rucio.daemons.conveyor.poller import poller
 from rucio.daemons.conveyor.preparer import preparer
@@ -86,6 +88,17 @@ def __wait_for_request_state(dst_rse_id, scope, name, state, max_wait_seconds=MA
     return request
 
 
+def __wait_for_fts_state(request, expected_state, max_wait_seconds=MAX_POLL_WAIT_SECONDS):
+    job_state = ''
+    for _ in range(max_wait_seconds):
+        fts_response = FTS3Transfertool(external_host=TEST_FTS_HOST).bulk_query(request['external_id'])
+        job_state = fts_response[request['external_id']][request['id']]['job_state']
+        if job_state == expected_state:
+            break
+        time.sleep(1)
+    return job_state
+
+
 def set_query_parameters(url, params):
     """
     Set a query parameter in an url which may, or may not, have other existing query parameters
@@ -99,6 +112,16 @@ def set_query_parameters(url, params):
     return urlunparse(url_parts)
 
 
+@read_session
+def __get_source(request_id, src_rse_id, scope, name, session=None):
+    return session.query(models.Source) \
+        .filter(models.Source.request_id == request_id) \
+        .filter(models.Source.scope == scope) \
+        .filter(models.Source.name == name) \
+        .filter(models.Source.rse_id == src_rse_id) \
+        .first()
+
+
 @skip_rse_tests_with_accounts
 @pytest.mark.dirty(reason="leaves files in XRD containers")
 @pytest.mark.noparallel(reason="uses predefined RSEs; runs submitter, poller and finisher; changes XRD3 usage and limits")
@@ -167,7 +190,11 @@ def test_multihop_intermediate_replica_lifecycle(vo, did_factory, root_account,
         replica = replica_core.get_replica(rse_id=jump_rse_id, **did)
         assert replica
 
-        # FTS fails the second transfer, so run submitter again to copy from jump rse to destination rse
+        # FTS fails the second transfer
+        request = __wait_for_request_state(dst_rse_id=dst_rse_id, state=RequestState.FAILED, **did)
+        # ensure tha the ranking was correctly decreased
+        assert __get_source(request_id=request['id'], src_rse_id=jump_rse_id, **did).ranking == -1
+        # run submitter again to copy from jump rse to destination rse
         submitter(once=True, rses=[{'id': rse_id} for rse_id in all_rses], partition_wait_time=None, transfertype='single', filter_transfertool=None)
 
         # Wait for the destination replica to become ready
@@ -226,6 +253,7 @@ def test_fts_non_recoverable_failures_handled_on_multihop(vo, did_factory, root_
     submitter(once=True, rses=[{'id': rse_id} for rse_id in all_rses], group_bulk=2, partition_wait_time=None, transfertype='single', filter_transfertool=None)
 
     request = __wait_for_request_state(dst_rse_id=dst_rse_id, state=RequestState.FAILED, **did)
+    assert 'Cancelled hop in multi-hop' in request['err_msg']
     assert request['state'] == RequestState.FAILED
     request = request_core.get_request_by_did(rse_id=jump_rse_id, **did)
     assert request['state'] == RequestState.FAILED
@@ -238,6 +266,9 @@ def test_fts_non_recoverable_failures_handled_on_multihop(vo, did_factory, root_
     with pytest.raises(RequestNotFound):
         request_core.get_request_by_did(rse_id=jump_rse_id, **did)
     request = request_core.get_request_by_did(rse_id=dst_rse_id, **did)
+    # ensure tha the ranking was correctly decreased for the whole path
+    assert __get_source(request_id=request['id'], src_rse_id=jump_rse_id, **did).ranking == -1
+    assert __get_source(request_id=request['id'], src_rse_id=src_rse_id, **did).ranking == -1
     assert request['state'] == RequestState.QUEUED
 
 
@@ -363,7 +394,7 @@ def test_multisource_receiver(vo, did_factory, replica_client, root_account, met
     Run receiver as a background thread to automatically handle fts notifications.
     Ensure that a multi-source job in which the first source fails is correctly handled by receiver.
     """
-    receiver_thread = threading.Thread(target=receiver, kwargs={'id': 0, 'full_mode': True, 'all_vos': True, 'total_threads': 1})
+    receiver_thread = threading.Thread(target=receiver, kwargs={'id_': 0, 'full_mode': True, 'all_vos': True, 'total_threads': 1})
     receiver_thread.start()
 
     try:
@@ -423,7 +454,7 @@ def test_multihop_receiver_on_failure(vo, did_factory, replica_client, root_acco
     """
     Verify that the receiver correctly handles multihop jobs which fail
     """
-    receiver_thread = threading.Thread(target=receiver, kwargs={'id': 0, 'full_mode': True, 'all_vos': True, 'total_threads': 1})
+    receiver_thread = threading.Thread(target=receiver, kwargs={'id_': 0, 'full_mode': True, 'all_vos': True, 'total_threads': 1})
     receiver_thread.start()
 
     try:
@@ -450,6 +481,7 @@ def test_multihop_receiver_on_failure(vo, did_factory, replica_client, root_acco
         # TODO: set the run_poller argument to False if we ever manage to make the receiver correctly handle multi-hop failures.
         request = __wait_for_request_state(dst_rse_id=dst_rse_id, state=RequestState.FAILED, run_poller=True, **did)
         assert request['state'] == RequestState.FAILED
+        assert 'Cancelled hop in multi-hop' in request['err_msg']
 
         # First hop will be handled by receiver; second hop by poller
         assert metrics_mock.get_sample_value('rucio_daemons_conveyor_receiver_update_request_state_total', labels={'updated': 'True'}) >= 1
@@ -460,6 +492,9 @@ def test_multihop_receiver_on_failure(vo, did_factory, replica_client, root_acco
         with pytest.raises(RequestNotFound):
             request_core.get_request_by_did(rse_id=jump_rse_id, **did)
         request = request_core.get_request_by_did(rse_id=dst_rse_id, **did)
+        # ensure tha the ranking was correctly decreased for the whole path
+        assert __get_source(request_id=request['id'], src_rse_id=jump_rse_id, **did).ranking == -1
+        assert __get_source(request_id=request['id'], src_rse_id=src_rse_id, **did).ranking == -1
         assert request['state'] == RequestState.QUEUED
     finally:
         receiver_graceful_stop.set()
@@ -480,7 +515,7 @@ def test_multihop_receiver_on_success(vo, did_factory, root_account, core_config
     """
     Verify that the receiver correctly handles successful multihop jobs
     """
-    receiver_thread = threading.Thread(target=receiver, kwargs={'id': 0, 'full_mode': True, 'all_vos': True, 'total_threads': 1})
+    receiver_thread = threading.Thread(target=receiver, kwargs={'id_': 0, 'full_mode': True, 'all_vos': True, 'total_threads': 1})
     receiver_thread.start()
 
     try:
@@ -522,7 +557,7 @@ def test_multihop_receiver_on_success(vo, did_factory, root_account, core_config
 @pytest.mark.parametrize("core_config_mock", [{
     "table_content": [('throttler', 'mode', 'DEST_PER_ALL_ACT')]
 }], indirect=True)
-def test_preparer_throttler_submitter(rse_factory, did_factory, root_account, file_config_mock, core_config_mock):
+def test_preparer_throttler_submitter(rse_factory, did_factory, root_account, file_config_mock, core_config_mock, metrics_mock):
     """
     Integration test of the preparer/throttler workflow.
     """
@@ -568,6 +603,12 @@ def test_preparer_throttler_submitter(rse_factory, did_factory, root_account, fi
 
     # One of the waiting requests will be queued, the second will remain in waiting state
     throttler(once=True, partition_wait_time=None)
+    # Check metrics.
+    # This gauge values are recorded at the beginning of the execution. Hence 2 waiting and 0 transfers
+    gauge_name = 'rucio_daemons_conveyor_throttler_set_rse_transfer_limits'
+    assert metrics_mock.get_sample_value(gauge_name, labels={'activity': 'all_activities', 'rse': dst_rse1, 'limit_attr': 'max_transfers'}) == 1
+    assert metrics_mock.get_sample_value(gauge_name, labels={'activity': 'all_activities', 'rse': dst_rse1, 'limit_attr': 'transfers'}) == 0
+    assert metrics_mock.get_sample_value(gauge_name, labels={'activity': 'all_activities', 'rse': dst_rse1, 'limit_attr': 'waiting'}) == 2
     request1 = request_core.get_request_by_did(rse_id=dst_rse_id1, **did1)
     request2 = request_core.get_request_by_did(rse_id=dst_rse_id1, **did2)
     # one request WAITING and other QUEUED
@@ -580,6 +621,10 @@ def test_preparer_throttler_submitter(rse_factory, did_factory, root_account, fi
 
     # Calling the throttler again will not schedule the waiting request, because there is a submitted one
     throttler(once=True, partition_wait_time=None)
+    # This gauge values are recorded at the beginning of the execution. Hence 1 waiting and one transfer
+    assert metrics_mock.get_sample_value(gauge_name, labels={'activity': 'all_activities', 'rse': dst_rse1, 'limit_attr': 'max_transfers'}) == 1
+    assert metrics_mock.get_sample_value(gauge_name, labels={'activity': 'all_activities', 'rse': dst_rse1, 'limit_attr': 'transfers'}) == 1
+    assert metrics_mock.get_sample_value(gauge_name, labels={'activity': 'all_activities', 'rse': dst_rse1, 'limit_attr': 'waiting'}) == 1
     request = request_core.get_request_by_did(rse_id=dst_rse_id1, **waiting_did)
     assert request['state'] == RequestState.WAITING
 
@@ -705,63 +750,298 @@ def test_lost_transfers(rse_factory, did_factory, root_account):
     # Run finisher and submitter, the request must be resubmitted and transferred correctly
     __update_request(request['id'], updated_at=datetime.utcnow() - timedelta(days=1))
     finisher(once=True, partition_wait_time=None)
+    # The source ranking must not be updated for submission failures and lost transfers
+    request = request_core.get_request_by_did(rse_id=dst_rse_id, **did)
+    assert __get_source(request_id=request['id'], src_rse_id=src_rse_id, **did).ranking == 0
     submitter(once=True, rses=[{'id': rse_id} for rse_id in all_rses], group_bulk=2, partition_wait_time=None, transfertype='single', filter_transfertool=None)
     replica = __wait_for_replica_transfer(dst_rse_id=dst_rse_id, **did)
     assert replica['state'] == ReplicaState.AVAILABLE
 
 
+class FTSWrapper(FTS3Transfertool):
+    """
+    Used to alter the JSON exchange with FTS.
+
+    One use-case would be to use the "mock" gfal plugin (by using a mock:// protocol/scheme) to simulate failuare on fts side.
+    For example, adding size_pre=<something> url parameter would result in "stat" calls on FTS side to return(simulate) this file size.
+    https://gitlab.cern.ch/dmc/gfal2/-/blob/master/src/plugins/mock/README_PLUGIN_MOCK
+    """
+
+    @staticmethod
+    def on_submit(file):
+        pass
+
+    @staticmethod
+    def on_receive(job_response):
+        pass
+
+    @classmethod
+    def _FTS3Transfertool__file_from_transfer(cls, transfer, job_params):
+        file = super()._FTS3Transfertool__file_from_transfer(transfer, job_params)
+        cls.on_submit(file)
+        return file
+
+    def _FTS3Transfertool__bulk_query_responses(self, jobs_response):
+        self.on_receive(jobs_response)
+        return super()._FTS3Transfertool__bulk_query_responses(jobs_response)
+
+
+@pytest.fixture
+def overwrite_on_tape_topology(rse_factory, did_factory, root_account, vo, file_factory):
+    """
+    Prepares the XRD* RSEs for an overwrite_on_tape test.
+    - fakes that one xroot RSE is a tape destination (and rollbacks the change after the test)
+
+    Return a factory which allows to upload/register/add_rule for two dids
+    """
+
+    rse1 = 'XRD1'
+    rse1_id = rse_core.get_rse_id(rse=rse1, vo=vo)
+    rse2 = 'XRD3'
+    rse2_id = rse_core.get_rse_id(rse=rse2, vo=vo)
+    rse3 = 'XRD4'
+    rse3_id = rse_core.get_rse_id(rse=rse3, vo=vo)
+
+    def __generate_and_upload_file(src_rse, dst_rse, simulate_dst_corrupted=False):
+        """
+        Create and upload real files to source and destination. Don't register it on destination. This way, fts will fail if overwrite = False
+
+        If simulate_dst_corrupted is True, will upload a different file to destination, to simulate that it is corrupted
+        """
+        local_file = file_factory.file_generator()
+        did = did_factory.random_did()
+        did_factory.upload_test_file(src_rse, path=local_file, **did)
+        did_factory.upload_client.upload(
+            [
+                {
+                    'path': file_factory.file_generator(size=3) if simulate_dst_corrupted else local_file,
+                    'rse': dst_rse,
+                    'did_scope': did['scope'].external,
+                    'did_name': did['name'],
+                    'no_register': True,
+                }
+            ]
+        )
+        return did
+
+    def __create_dids(did1_corrupted=True, did2_corrupted=True):
+        """
+        Uploads two files:
+        - one which requires multiple transfer hop to go to destination
+        - one which can be transferred in one hop to destination rse
+        """
+        # multihop transfer:
+        did1 = __generate_and_upload_file(rse1, rse3, simulate_dst_corrupted=did1_corrupted)
+        # direct transfer
+        did2 = __generate_and_upload_file(rse2, rse3, simulate_dst_corrupted=did2_corrupted)
+        rule_core.add_rule(dids=[did1, did2], account=root_account, copies=1, rse_expression=rse3, grouping='ALL', weight=None, lifetime=None, locked=False, subscription_id=None)
+
+        return rse1_id, rse2_id, rse3_id, did1, did2
+
+    # Fake that destination RSE is a tape
+    rse_core.update_rse(rse_id=rse3_id, parameters={'rse_type': RSEType.TAPE})
+    try:
+        rse_core.add_rse_attribute(rse3_id, 'archive_timeout', 60)
+        yield __create_dids
+    finally:
+        rse_core.update_rse(rse_id=rse3_id, parameters={'rse_type': RSEType.DISK})
+        rse_core.del_rse_attribute(rse3_id, 'archive_timeout')
+
+
 @skip_rse_tests_with_accounts
-@pytest.mark.noparallel(reason="runs submitter and poller;")
+@pytest.mark.dirty(reason="leaves files in XRD containers")
+@pytest.mark.noparallel(reason="runs submitter; poller and finisher")
 @pytest.mark.parametrize("core_config_mock", [{"table_content": [
     ('transfers', 'use_multihop', True)
 ]}], indirect=True)
 @pytest.mark.parametrize("caches_mock", [{"caches_to_mock": [
+    'rucio.common.rse_attributes.REGION',
+    'rucio.core.rse.REGION',
     'rucio.core.rse_expression_parser.REGION',  # The list of multihop RSEs is retrieved by an expression
     'rucio.core.config.REGION',
+    'rucio.rse.rsemanager.RSE_REGION',  # for RSE info
 ]}], indirect=True)
-def test_overwrite_on_tape(rse_factory, did_factory, root_account, core_config_mock, caches_mock):
+def test_overwrite_on_tape(overwrite_on_tape_topology, core_config_mock, caches_mock):
     """
     Ensure that overwrite is not set for transfers towards TAPE RSEs
-
-    Submit a real transfer to FTS and rely on the gfal "mock" plugin to trigger a failure. The failure is triggered
-    when gfal_stat is called on the destination URL and it returns a result. To achieve this via the mock
-    plugin, it's enough to have a mock:// protocol/scheme and add size_pre=<something> url parameter.
-    https://gitlab.cern.ch/dmc/gfal2/-/blob/master/src/plugins/mock/README_PLUGIN_MOCK
     """
-    # +------+    +------+    +------+
-    # |      |    |      |    |      |
-    # | RSE1 +--->| RSE2 |--->| RSE3 |
-    # |      |    |      |    |(tape)|
-    # +------+    +------+    +------+
-    rse1, rse1_id = rse_factory.make_rse(scheme='mock', protocol_impl='rucio.rse.protocols.posix.Default')
-    rse2, rse2_id = rse_factory.make_rse(scheme='mock', protocol_impl='rucio.rse.protocols.posix.Default')
-    rse3, rse3_id = rse_factory.make_rse(scheme='mock', protocol_impl='rucio.rse.protocols.posix.Default', rse_type=RSEType.TAPE)
+    rse1_id, rse2_id, rse3_id, did1, did2 = overwrite_on_tape_topology(did1_corrupted=False, did2_corrupted=True)
     all_rses = [rse1_id, rse2_id, rse3_id]
 
-    distance_core.add_distance(rse1_id, rse2_id, ranking=10)
-    distance_core.add_distance(rse2_id, rse3_id, ranking=10)
-    rse_core.add_rse_attribute(rse2_id, 'available_for_multihop', True)
-    for rse_id in all_rses:
-        rse_core.add_rse_attribute(rse_id, 'fts', TEST_FTS_HOST)
-
-    # multihop transfer:
-    did1 = did_factory.upload_test_file(rse1)
-    # direct transfer:
-    did2 = did_factory.upload_test_file(rse2)
-
-    rule_core.add_rule(dids=[did1, did2], account=root_account, copies=1, rse_expression=rse3, grouping='ALL', weight=None, lifetime=None, locked=False, subscription_id=None)
-
-    # Wrap dest url generation to add size_pre=2 query parameter
-    non_mocked_generate_dest_url = transfer_core.DirectTransferDefinition._generate_dest_url
-
-    def mocked_generate_dest_url(cls, *args):
-        return set_query_parameters(non_mocked_generate_dest_url(*args), {'size_pre': 2})
-
-    with patch('rucio.core.transfer.DirectTransferDefinition._generate_dest_url', new=mocked_generate_dest_url):
-        submitter(once=True, rses=[{'id': rse_id} for rse_id in all_rses], group_bulk=10, partition_wait_time=None, transfertype='single', filter_transfertool=None)
+    submitter(once=True, rses=[{'id': rse_id} for rse_id in all_rses], group_bulk=10, partition_wait_time=None, transfertype='single', filter_transfertool=None)
 
     request = __wait_for_request_state(dst_rse_id=rse3_id, state=RequestState.FAILED, **did1)
     assert request['state'] == RequestState.FAILED
+    assert 'Destination file exists and overwrite is not enabled' in request['err_msg']
     request = __wait_for_request_state(dst_rse_id=rse3_id, state=RequestState.FAILED, **did2)
     assert request['state'] == RequestState.FAILED
     assert 'Destination file exists and overwrite is not enabled' in request['err_msg']
+
+
+@skip_rse_tests_with_accounts
+@pytest.mark.dirty(reason="leaves files in XRD containers")
+@pytest.mark.noparallel(reason="runs submitter; poller and finisher")
+@pytest.mark.parametrize("core_config_mock", [{"table_content": [
+    ('transfers', 'use_multihop', True)
+]}], indirect=True)
+@pytest.mark.parametrize("caches_mock", [{"caches_to_mock": [
+    'rucio.common.rse_attributes.REGION',
+    'rucio.core.rse.REGION',
+    'rucio.core.rse_expression_parser.REGION',  # The list of multihop RSEs is retrieved by an expression
+    'rucio.core.config.REGION',
+    'rucio.rse.rsemanager.RSE_REGION',  # for RSE info
+]}], indirect=True)
+def test_file_exists_handled(overwrite_on_tape_topology, core_config_mock, caches_mock):
+    """
+    If a transfer fails because the destination job_params exists, and the size+checksums of that existing job_params
+    are correct, the transfer must be marked successful.
+    """
+    rse1_id, rse2_id, rse3_id, did1, did2 = overwrite_on_tape_topology(did1_corrupted=False, did2_corrupted=False)
+    all_rses = [rse1_id, rse2_id, rse3_id]
+
+    class _FTSWrapper(FTSWrapper):
+        @staticmethod
+        def on_receive(job_params):
+            for job in (job_params if isinstance(job_params, list) else [job_params]):
+                for file in job.get('files', []):
+                    if (file.get('file_metadata', {}).get('dst_type') == 'TAPE'
+                            and file.get('file_metadata', {}).get('dst_file', {}).get('file_on_tape') is not None):
+                        # Fake that dst_file metadata contains file_on_tape == True
+                        # As we don't really have tape RSEs in our tests, file_on_tape is always false
+                        file['file_metadata']['dst_file']['file_on_tape'] = True
+            return job_params
+
+    with patch('rucio.core.transfer.FTS3Transfertool', _FTSWrapper):
+        submitter(once=True, rses=[{'id': rse_id} for rse_id in all_rses], group_bulk=10, partition_wait_time=None, transfertype='single', filter_transfertool=None)
+
+        request = __wait_for_request_state(dst_rse_id=rse3_id, state=RequestState.DONE, **did1)
+        assert request['state'] == RequestState.DONE
+        request = __wait_for_request_state(dst_rse_id=rse3_id, state=RequestState.DONE, **did2)
+        assert request['state'] == RequestState.DONE
+
+
+@skip_rse_tests_with_accounts
+@pytest.mark.dirty(reason="leaves files in XRD containers; leaves pending fts transfers in archiving state")
+@pytest.mark.noparallel(reason="runs submitter; poller and finisher")
+@pytest.mark.parametrize("core_config_mock", [{"table_content": [
+    ('transfers', 'use_multihop', True),
+    ('transfers', 'overwrite_corrupted_files', False)
+]}], indirect=True)
+@pytest.mark.parametrize("caches_mock", [{"caches_to_mock": [
+    'rucio.common.rse_attributes.REGION',
+    'rucio.core.rse.REGION',
+    'rucio.core.rse_expression_parser.REGION',  # The list of multihop RSEs is retrieved by an expression
+    'rucio.core.config.REGION',
+    'rucio.rse.rsemanager.RSE_REGION',  # for RSE info
+]}], indirect=True)
+def test_overwrite_corrupted_files(overwrite_on_tape_topology, core_config_mock, caches_mock):
+    """
+    If a transfer fails because the destination exists, and the size+checksums of the destination file are wrong,
+    the next submission must be performed according to the overwrite_corrupted_files config paramenter.
+    """
+    rse1_id, rse2_id, rse3_id, did1, did2 = overwrite_on_tape_topology(did1_corrupted=True, did2_corrupted=True)
+    all_rses = [rse1_id, rse2_id, rse3_id]
+
+    class _FTSWrapper(FTSWrapper):
+        @staticmethod
+        def on_receive(job_params):
+            for job in (job_params if isinstance(job_params, list) else [job_params]):
+                for file in job.get('files', []):
+                    if (file.get('file_metadata', {}).get('dst_type') == 'TAPE'
+                            and file.get('file_metadata', {}).get('dst_file', {}).get('file_on_tape') is not None):
+                        # Fake that dst_file metadata contains file_on_tape == True
+                        # As we don't really have tape RSEs in our tests, file_on_tape is always false
+                        file['file_metadata']['dst_file']['file_on_tape'] = True
+            return job_params
+
+    with patch('rucio.core.transfer.FTS3Transfertool', _FTSWrapper):
+        submitter(once=True, rses=[{'id': rse_id} for rse_id in all_rses], group_bulk=10, partition_wait_time=None, transfertype='single', filter_transfertool=None)
+        # Both transfers must be marked as failed because the file size is incorrect
+        request = __wait_for_request_state(dst_rse_id=rse3_id, state=RequestState.FAILED, **did1)
+        assert request['state'] == RequestState.FAILED
+        request = __wait_for_request_state(dst_rse_id=rse3_id, state=RequestState.FAILED, **did2)
+        assert request['state'] == RequestState.FAILED
+
+        # Re-submit the failed requests. They must fail again, because overwrite_corrupted_files is False
+        finisher(once=True, partition_wait_time=None)
+        request = request_core.get_request_by_did(rse_id=rse3_id, **did1)
+        assert request['state'] == RequestState.QUEUED
+        request = request_core.get_request_by_did(rse_id=rse3_id, **did2)
+        assert request['state'] == RequestState.QUEUED
+        submitter(once=True, rses=[{'id': rse_id} for rse_id in all_rses], group_bulk=10, partition_wait_time=None, transfertype='single', filter_transfertool=None)
+        # Set overwrite to True before running the poller or finisher
+        core_config.set('transfers', 'overwrite_corrupted_files', True)
+        request = __wait_for_request_state(dst_rse_id=rse3_id, state=RequestState.FAILED, **did1)
+        assert request['state'] == RequestState.FAILED
+        request = __wait_for_request_state(dst_rse_id=rse3_id, state=RequestState.FAILED, **did2)
+        assert request['state'] == RequestState.FAILED
+
+        # Re-submit one more time. Now the destination file must be overwritten
+        finisher(once=True, partition_wait_time=None)
+        request = request_core.get_request_by_did(rse_id=rse3_id, **did1)
+        assert request['state'] == RequestState.QUEUED
+        request = request_core.get_request_by_did(rse_id=rse3_id, **did2)
+        assert request['state'] == RequestState.QUEUED
+        submitter(once=True, rses=[{'id': rse_id} for rse_id in all_rses], group_bulk=10, partition_wait_time=None, transfertype='single', filter_transfertool=None)
+        request = request_core.get_request_by_did(rse_id=rse3_id, **did1)
+        assert request['state'] == RequestState.SUBMITTED
+        assert __wait_for_fts_state(request, expected_state='ARCHIVING') == 'ARCHIVING'
+        request = request_core.get_request_by_did(rse_id=rse3_id, **did2)
+        assert request['state'] == RequestState.SUBMITTED
+        assert __wait_for_fts_state(request, expected_state='ARCHIVING') == 'ARCHIVING'
+
+
+@pytest.mark.noparallel(reason="runs submitter; poller and finisher")
+@pytest.mark.parametrize("file_config_mock", [{"overrides": [
+    ('conveyor', 'usercert', 'DEFAULT_DUMMY_CERT'),
+    ('vo_certs', 'new', 'NEW_VO_DUMMY_CERT'),
+]}], indirect=True)
+def test_multi_vo_certificates(file_config_mock, rse_factory, did_factory, scope_factory, vo, second_vo):
+    """
+    Test that submitter and poller call fts with correct certificates in multi-vo env
+    """
+
+    _, [scope1, scope2] = scope_factory(vos=[vo, second_vo])
+
+    def __init_test_for_vo(vo, scope):
+        src_rse, src_rse_id = rse_factory.make_rse(scheme='mock', protocol_impl='rucio.rse.protocols.posix.Default', vo=vo)
+        dst_rse, dst_rse_id = rse_factory.make_rse(scheme='mock', protocol_impl='rucio.rse.protocols.posix.Default', vo=vo)
+        all_rses = [src_rse_id, dst_rse_id]
+
+        for rse_id in all_rses:
+            rse_core.add_rse_attribute(rse_id, 'fts', TEST_FTS_HOST)
+        distance_core.add_distance(src_rse_id, dst_rse_id, ranking=10)
+        account = InternalAccount('root', vo=vo)
+        did = did_factory.random_did(scope=scope)
+        replica_core.add_replica(rse_id=src_rse_id, scope=scope, name=did['name'], bytes_=1, account=account, adler32=None, md5=None)
+        rule_core.add_rule(dids=[did], account=account, copies=1, rse_expression=dst_rse, grouping='ALL', weight=None,
+                           lifetime=None, locked=False, subscription_id=None, ignore_account_limit=True)
+        return all_rses
+
+    all_rses = []
+    rses = __init_test_for_vo(vo=vo, scope=scope1)
+    all_rses.extend(rses)
+    rses = __init_test_for_vo(vo=second_vo, scope=scope2)
+    all_rses.extend(rses)
+
+    certs_used_by_submitter = []
+    certs_used_by_poller = []
+
+    class _FTSWrapper(FTS3Transfertool):
+        # Override fts3 transfertool. Don't actually perform any interaction with fts; and record the certificates used
+        def submit(self, transfers, job_params, timeout=None):
+            certs_used_by_submitter.append(self.cert[0])
+            return generate_uuid()
+
+        def bulk_query(self, transfer_ids, timeout=None):
+            certs_used_by_poller.append(self.cert[0])
+            return {}
+
+    with patch('rucio.daemons.conveyor.submitter.TRANSFERTOOL_CLASSES_BY_NAME') as tt_mock:
+        tt_mock.__getitem__.return_value = _FTSWrapper
+        submitter(once=True, rses=[{'id': rse_id} for rse_id in all_rses], group_bulk=2, partition_wait_time=None, transfertype='single', filter_transfertool=None)
+        assert sorted(certs_used_by_submitter) == ['DEFAULT_DUMMY_CERT', 'NEW_VO_DUMMY_CERT']
+
+    with patch('rucio.core.transfer.FTS3Transfertool', _FTSWrapper):
+        poller(once=True, older_than=0, partition_wait_time=None)
+        assert sorted(certs_used_by_poller) == ['DEFAULT_DUMMY_CERT', 'NEW_VO_DUMMY_CERT']
