@@ -31,6 +31,7 @@
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
 # - Sahan Dilshan <32576163+sahandilshan@users.noreply.github.com>, 2021
 # - Radu Carpa <radu.carpa@cern.ch>, 2021
+# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
 
 from __future__ import absolute_import, division
 import datetime
@@ -57,10 +58,9 @@ from requests.packages.urllib3 import disable_warnings  # pylint: disable=import
 
 from dogpile.cache import make_region
 from dogpile.cache.api import NoValue
-from prometheus_client import Summary
 
 from rucio.common.config import config_get, config_get_bool
-from rucio.common.constants import FTS_STATE
+from rucio.common.constants import FTS_JOB_TYPE, FTS_STATE
 from rucio.common.exception import TransferToolTimeout, TransferToolWrongAnswer, DuplicateFileTransferSubmission
 from rucio.common.utils import APIEncoder, chunks, set_checksum_value
 from rucio.core.rse import get_rse_supported_checksums_from_attributes
@@ -86,13 +86,10 @@ WHOAMI_COUNTER = MultiCounter(prom='rucio_transfertool_fts3_whoami', statsd='tra
                               documentation='Number of whoami requests', labelnames=('state', 'host'))
 VERSION_COUNTER = MultiCounter(prom='rucio_transfertool_fts3_version', statsd='transfertool.fts3.{host}.version.{state}',
                                documentation='Number of version requests', labelnames=('state', 'host'))
-QUERY_LATEST_COUNTER = MultiCounter(prom='rucio_transfertool_fts3_query_latest', statsd='transfertool.fts3.{host}.query_latest.{state}',
-                                    documentation='Number of latest status queries', labelnames=('state', 'host'))
 BULK_QUERY_COUNTER = MultiCounter(prom='rucio_transfertool_fts3_bulk_query', statsd='transfertool.fts3.{host}.bulk_query.{state}',
                                   documentation='Number of bulk queries', labelnames=('state', 'host'))
 QUERY_DETAILS_COUNTER = MultiCounter(prom='rucio_transfertool_fts3_query_details', statsd='transfertool.fts3.{host}.query_details.{state}',
                                      documentation='Number of detailed status queries', labelnames=('state', 'host'))
-SUBMISSION_TIMER = Summary('rucio_transfertool_fts3_submit_transfer', 'Timer for transfer submission', labelnames=('host',))
 
 
 ALLOW_USER_OIDC_TOKENS = config_get('conveyor', 'allow_user_oidc_tokens', False, False)
@@ -355,7 +352,7 @@ class FTS3Transfertool(Transfertool):
     FTS3 implementation of a Rucio transfertool
     """
 
-    def __init__(self, external_host, oidc_account=None, group_bulk=1, group_policy='rule', source_strategy=None,
+    def __init__(self, external_host, oidc_account=None, vo=None, group_bulk=1, group_policy='rule', source_strategy=None,
                  max_time_in_queue=None, bring_online=43200, default_lifetime=172800, archive_timeout_override=None,
                  logger=logging.log):
         """
@@ -375,6 +372,8 @@ class FTS3Transfertool(Transfertool):
         self.archive_timeout_override = archive_timeout_override
 
         usercert = config_get('conveyor', 'usercert', False, None)
+        if vo:
+            usercert = config_get('vo_certs', vo, False, usercert)
 
         # token for OAuth 2.0 OIDC authorization scheme (working only with dCache + davs/https protocols as of Sep 2019)
         self.token = None
@@ -405,8 +404,12 @@ class FTS3Transfertool(Transfertool):
             self.cert = None
             self.verify = True  # True is the default setting of a requests.* method
 
-    @staticmethod
-    def submission_builder_for_path(transfer_path, logger=logging.log):
+    @classmethod
+    def submission_builder_for_path(cls, transfer_path, logger=logging.log):
+        vo = None
+        if config_get_bool('common', 'multi_vo', False, None):
+            vo = transfer_path[-1].rws.scope.vo
+
         all_hops_have_an_fts_attr = True
         fts_hosts = []
         for hop in transfer_path:
@@ -428,7 +431,7 @@ class FTS3Transfertool(Transfertool):
             logger(logging.DEBUG, 'OAuth2/OIDC available for transfer {}'.format([str(hop) for hop in transfer_path]))
             oidc_account = transfer_path[-1].rws.account
 
-        return TransferToolBuilder(FTS3Transfertool, external_host=fts_hosts[0], oidc_account=oidc_account)
+        return TransferToolBuilder(cls, external_host=fts_hosts[0], oidc_account=oidc_account, vo=vo)
 
     def group_into_submit_jobs(self, transfer_paths):
         jobs = bulk_group_transfers(
@@ -524,7 +527,7 @@ class FTS3Transfertool(Transfertool):
             job_params["id_generator"] = "deterministic"
             job_params["sid"] = files[0]['metadata']['request_id']
             expected_transfer_id = self.__get_deterministic_id(job_params["sid"])
-            logging.debug("Submit bulk transfers in deterministic mode, sid %s, expected transfer id: %s", job_params["sid"], expected_transfer_id)
+            self.logger(logging.DEBUG, "Submit bulk transfers in deterministic mode, sid %s, expected transfer id: %s", job_params["sid"], expected_transfer_id)
 
         # bulk submission
         params_dict = {'files': files, 'params': job_params}
@@ -539,15 +542,14 @@ class FTS3Transfertool(Transfertool):
                                         data=params_str,
                                         headers=self.headers,
                                         timeout=timeout)
-            record_timer('transfertool.fts3.submit_transfer.%s' % self.__extract_host(self.external_host), (time.time() - start_time) * 1000 / len(files))
             labels = {'host': self.__extract_host(self.external_host)}
-            SUBMISSION_TIMER.labels(**labels).observe((time.time() - start_time) * 1000 / len(files))
+            record_timer('transfertool.fts3.submit_transfer.{host}', (time.time() - start_time) * 1000 / len(files), labels=labels)
         except ReadTimeout as error:
             raise TransferToolTimeout(error)
         except JSONDecodeError as error:
             raise TransferToolWrongAnswer(error)
         except Exception as error:
-            logging.warning('Could not submit transfer to %s - %s' % (self.external_host, str(error)))
+            self.logger(logging.WARNING, 'Could not submit transfer to %s - %s' % (self.external_host, str(error)))
 
         if post_result and post_result.status_code == 200:
             SUBMISSION_COUNTER.labels(state='success', host=self.__extract_host(self.external_host)).inc(len(files))
@@ -558,9 +560,9 @@ class FTS3Transfertool(Transfertool):
         else:
             if expected_transfer_id:
                 transfer_id = expected_transfer_id
-                logging.warning("Failed to submit transfer to %s, will use expected transfer id %s, error: %s", self.external_host, transfer_id, post_result.text if post_result is not None else post_result)
+                self.logger(logging.WARNING, "Failed to submit transfer to %s, will use expected transfer id %s, error: %s", self.external_host, transfer_id, post_result.text if post_result is not None else post_result)
             else:
-                logging.warning("Failed to submit transfer to %s, error: %s", self.external_host, post_result.text if post_result is not None else post_result)
+                self.logger(logging.WARNING, "Failed to submit transfer to %s, error: %s", self.external_host, post_result.text if post_result is not None else post_result)
             SUBMISSION_COUNTER.labels(state='failure', host=self.__extract_host(self.external_host)).inc(len(files))
 
         if not transfer_id:
@@ -699,58 +701,6 @@ class FTS3Transfertool(Transfertool):
         VERSION_COUNTER.labels(state='failure', host=self.__extract_host(self.external_host)).inc()
         raise Exception('Could not retrieve version: %s', get_result.content)
 
-    def query_latest(self, state, last_nhours=1):
-        """
-        Query the latest status transfers status in FTS3 via JSON.
-
-        :param state: Transfer state as a string or a dictionary.
-        :returns: Transfer status information as a dictionary.
-        """
-
-        jobs = None
-
-        try:
-            whoami = requests.get('%s/whoami' % (self.external_host),
-                                  verify=self.verify,
-                                  cert=self.cert,
-                                  headers=self.headers)
-            if whoami and whoami.status_code == 200:
-                delegation_id = whoami.json()['delegation_id']
-            else:
-                raise Exception('Could not retrieve delegation id: %s', whoami.content)
-            state_string = ','.join(state)
-            jobs = requests.get('%s/jobs?dlg_id=%s&state_in=%s&time_window=%s' % (self.external_host,
-                                                                                  delegation_id,
-                                                                                  state_string,
-                                                                                  last_nhours),
-                                verify=self.verify,
-                                cert=self.cert,
-                                headers=self.headers)
-        except ReadTimeout as error:
-            raise TransferToolTimeout(error)
-        except JSONDecodeError as error:
-            raise TransferToolWrongAnswer(error)
-        except Exception:
-            logging.warning('Could not query latest terminal states from %s', self.external_host)
-
-        if jobs and (jobs.status_code == 200 or jobs.status_code == 207):
-            record_counter('transfertool.fts3.%s.query_latest.success' % self.__extract_host(self.external_host))
-            labels = {'state': 'success', 'host': self.__extract_host(self.external_host)}
-            QUERY_LATEST_COUNTER.labels(**labels).inc()
-            try:
-                jobs_json = jobs.json()
-                return jobs_json
-            except ReadTimeout as error:
-                raise TransferToolTimeout(error)
-            except JSONDecodeError as error:
-                raise TransferToolWrongAnswer(error)
-            except Exception as error:
-                logging.error("Failed to parse the jobs status %s" % (str(error)))
-
-        record_counter('transfertool.fts3.%s.query.failure' % self.__extract_host(self.external_host))
-        labels = {'state': 'failure', 'host': self.__extract_host(self.external_host)}
-        QUERY_LATEST_COUNTER.labels(**labels).inc()
-
     def bulk_query(self, transfer_ids, timeout=None):
         """
         Query the status of a bulk of transfers in FTS3 via JSON.
@@ -828,7 +778,7 @@ class FTS3Transfertool(Transfertool):
                                   headers=self.headers,
                                   timeout=None)
         except Exception:
-            logging.warning('Could not get config of %s on %s - %s', storage_element, self.external_host, str(traceback.format_exc()))
+            self.logger(logging.WARNING, 'Could not get config of %s on %s - %s', storage_element, self.external_host, str(traceback.format_exc()))
         if result and result.status_code == 200:
             C = result.json()
             config_se = C[storage_element]
@@ -838,13 +788,14 @@ class FTS3Transfertool(Transfertool):
     def set_se_config(self, storage_element, inbound_max_active=None, outbound_max_active=None, inbound_max_throughput=None, outbound_max_throughput=None, staging=None):
         """
         Set the configuration for a storage element. Used for alleviating transfer failures due to timeout.
-        :returns: JSON post response in case of success, otherwise raise Exception.
+
         :param storage_element: The storage element to be configured
         :param inbound_max_active: the integer to set the inbound_max_active for the SE.
         :param outbound_max_active: the integer to set the outbound_max_active for the SE.
         :param inbound_max_throughput: the float to set the inbound_max_throughput for the SE.
         :param outbound_max_throughput: the float to set the outbound_max_throughput for the SE.
         :param staging: the integer to set the staging for the operation of a SE.
+        :returns: JSON post response in case of success, otherwise raise Exception.
         """
 
         params_dict = {storage_element: {'operations': {}, 'se_info': {}}}
@@ -852,7 +803,7 @@ class FTS3Transfertool(Transfertool):
             try:
                 policy = config_get('policy', 'permission')
             except Exception:
-                logging.warning('Could not get policy from config')
+                self.logger(logging.WARNING, 'Could not get policy from config')
             params_dict[storage_element]['operations'] = {policy: {'staging': staging}}
         # A lot of try-excepts to avoid dictionary overwrite's,
         # see https://stackoverflow.com/questions/27118687/updating-nested-dictionaries-when-data-has-existing-key/27118776
@@ -888,7 +839,7 @@ class FTS3Transfertool(Transfertool):
                                    timeout=None)
 
         except Exception:
-            logging.warning('Could not set the config of %s on %s - %s', storage_element, self.external_host, str(traceback.format_exc()))
+            self.logger(logging.WARNING, 'Could not set the config of %s on %s - %s', storage_element, self.external_host, str(traceback.format_exc()))
         if result and result.status_code == 200:
             configSe = result.json()
             return configSe
@@ -926,7 +877,7 @@ class FTS3Transfertool(Transfertool):
                                        headers=self.headers,
                                        timeout=None)
             except Exception:
-                logging.warning('Could not ban %s on %s - %s', storage_element, self.external_host, str(traceback.format_exc()))
+                self.logger(logging.WARNING, 'Could not ban %s on %s - %s', storage_element, self.external_host, str(traceback.format_exc()))
             if result and result.status_code == 200:
                 return 0
             raise Exception('Could not ban the storage %s , status code returned : %s', (storage_element, result.status_code if result else None))
@@ -940,7 +891,7 @@ class FTS3Transfertool(Transfertool):
                                          headers=self.headers,
                                          timeout=None)
             except Exception:
-                logging.warning('Could not unban %s on %s - %s', storage_element, self.external_host, str(traceback.format_exc()))
+                self.logger(logging.WARNING, 'Could not unban %s on %s - %s', storage_element, self.external_host, str(traceback.format_exc()))
             if result and result.status_code == 204:
                 return 0
             raise Exception('Could not unban the storage %s , status code returned : %s', (storage_element, result.status_code if result else None))
@@ -963,7 +914,7 @@ class FTS3Transfertool(Transfertool):
             key = 'voname: %s' % self.external_host
             result = REGION_SHORT.get(key)
             if isinstance(result, NoValue):
-                logging.debug("Refresh transfer baseid and voname for %s", self.external_host)
+                self.logger(logging.DEBUG, "Refresh transfer baseid and voname for %s", self.external_host)
 
                 get_result = None
                 try:
@@ -977,7 +928,7 @@ class FTS3Transfertool(Transfertool):
                 except JSONDecodeError as error:
                     raise TransferToolWrongAnswer(error)
                 except Exception as error:
-                    logging.warning('Could not get baseid and voname from %s - %s' % (self.external_host, str(error)))
+                    self.logger(logging.WARNING, 'Could not get baseid and voname from %s - %s' % (self.external_host, str(error)))
 
                 if get_result and get_result.status_code == 200:
                     baseid = str(get_result.json()['base_id'])
@@ -986,12 +937,12 @@ class FTS3Transfertool(Transfertool):
 
                     REGION_SHORT.set(key, result)
 
-                    logging.debug("Get baseid %s and voname %s from %s", baseid, voname, self.external_host)
+                    self.logger(logging.DEBUG, "Get baseid %s and voname %s from %s", baseid, voname, self.external_host)
                 else:
-                    logging.warning("Failed to get baseid and voname from %s, error: %s", self.external_host, get_result.text if get_result is not None else get_result)
+                    self.logger(logging.WARNING, "Failed to get baseid and voname from %s, error: %s", self.external_host, get_result.text if get_result is not None else get_result)
                     result = (None, None)
         except Exception as error:
-            logging.warning("Failed to get baseid and voname from %s: %s" % (self.external_host, str(error)))
+            self.logger(logging.WARNING, "Failed to get baseid and voname from %s: %s" % (self.external_host, str(error)))
             result = (None, None)
         return result
 
@@ -1021,10 +972,12 @@ class FTS3Transfertool(Transfertool):
 
         resps = {}
         multi_sources = fts_job_response['job_metadata'].get('multi_sources', False)
+        multi_hop = fts_job_response.get('job_type') == FTS_JOB_TYPE.MULTI_HOP
         for file_resp in fts_files_response:
+            file_state = file_resp['file_state']
             # for multiple source replicas jobs, the file_metadata(request_id) will be the same.
             # The next used file will overwrite the current used one. Only the last used file will return.
-            if multi_sources and file_resp['file_state'] == 'NOT_USED':
+            if multi_sources and file_state == FTS_STATE.NOT_USED:
                 continue
 
             if file_resp['start_time'] is None or file_resp['finish_time'] is None:
@@ -1034,11 +987,14 @@ class FTS3Transfertool(Transfertool):
                             - datetime.datetime.strptime(file_resp['start_time'], '%Y-%m-%dT%H:%M:%S')).seconds  # NOQA: W503
 
             request_id = file_resp['file_metadata']['request_id']
+            reason = file_resp.get('reason', None)
+            if not reason and file_state == FTS_STATE.NOT_USED and multi_hop:
+                reason = 'Cancelled hop in multi-hop'
             resps[request_id] = {'new_state': None,
                                  'transfer_id': fts_job_response.get('job_id'),
                                  'job_state': fts_job_response.get('job_state', None),
                                  'priority': fts_job_response.get('priority', None),
-                                 'file_state': file_resp.get('file_state', None),
+                                 'file_state': file_state,
                                  'src_url': file_resp.get('source_surl', None),
                                  'dst_url': file_resp.get('dest_surl', None),
                                  'started_at': datetime.datetime.strptime(file_resp['start_time'], '%Y-%m-%dT%H:%M:%S') if file_resp['start_time'] else None,
@@ -1046,13 +1002,14 @@ class FTS3Transfertool(Transfertool):
                                  'staging_finished': datetime.datetime.strptime(file_resp['staging_finished'], '%Y-%m-%dT%H:%M:%S') if file_resp['staging_finished'] else None,
                                  'transferred_at': datetime.datetime.strptime(file_resp['finish_time'], '%Y-%m-%dT%H:%M:%S') if file_resp['finish_time'] else None,
                                  'duration': duration,
-                                 'reason': file_resp.get('reason', None),
+                                 'reason': reason,
                                  'scope': file_resp['file_metadata'].get('scope', None),
                                  'name': file_resp['file_metadata'].get('name', None),
                                  'src_type': file_resp['file_metadata'].get('src_type', None),
                                  'dst_type': file_resp['file_metadata'].get('dst_type', None),
                                  'src_rse': file_resp['file_metadata'].get('src_rse', None),
                                  'dst_rse': file_resp['file_metadata'].get('dst_rse', None),
+                                 'dst_file': file_resp['file_metadata'].get('dst_file', {}),
                                  'request_id': file_resp['file_metadata'].get('request_id', None),
                                  'activity': file_resp['file_metadata'].get('activity', None),
                                  'src_rse_id': file_resp['file_metadata'].get('src_rse_id', None),
@@ -1066,7 +1023,7 @@ class FTS3Transfertool(Transfertool):
                                  'details': {'files': file_resp['file_metadata']}}
 
             # multiple source replicas jobs and we found the successful one, it's the final state.
-            if multi_sources and file_resp['file_state'] in [FTS_STATE.FINISHED]:
+            if multi_sources and file_state in [FTS_STATE.FINISHED]:
                 break
         return resps
 
